@@ -272,10 +272,29 @@ class ContentMigrationFirstStage(Stage):
                     extra_info = None
                     if is_multi_artifact:
                         extra_info = pulp_2to3_detail_content.get_treeinfo_serialized()
+                        # If we can't find the .treeinfo for the Distribution, warn and skip
+                        if extra_info is None:
+                            _logger.warning(
+                                _(
+                                    "Failed to find or instantiate extra_info for multi-artifact "
+                                    "pulp2 unit_id: {} ; skipping".format(pulp2content.pulp2_id)
+                                )
+                            )
+                            continue
                 else:
                     # create pulp3 content and assign relations if present
                     pulp3content, extra_info = pulp_2to3_detail_content.create_pulp3_content()
 
+                # If we can't find/create the Distribution, warn and skip
+                if pulp3content is None:
+                    _logger.warning(
+                        _(
+                            "Failed to find or instantiate pulp3 content for pulp2 unit_id: {} ;"
+                            " skipping".format(pulp2content.pulp2_id)
+                        )
+                    )
+                    continue
+
                 future_relations = {'pulp2content': pulp2content}
                 if extra_info:
                     future_relations.update(extra_info)
@@ -284,6 +303,8 @@ class ContentMigrationFirstStage(Stage):
                     d_artifacts = []
                     base_path = pulp2content.pulp2_storage_path
                     remotes = set()
+                    missing_artifact = False
+
                     for image_relative_path in extra_info['download']['images']:
                         image_path = os.path.join(base_path, image_relative_path)
                         downloaded = os.path.exists(image_path)
@@ -317,11 +338,8 @@ class ContentMigrationFirstStage(Stage):
                                 remote_declarative_artifacts.append(da)
 
                             if not remote_declarative_artifacts:
-                                _logger.warn(_(
-                                    'On_demand content cannot be migrated without a remote '
-                                    'pulp2 unit_id: {}'.format(pulp2content.pulp2_id))
-                                )
-                                continue
+                                missing_artifact = True
+                                break
 
                             d_artifacts.extend(remote_declarative_artifacts)
                         else:
@@ -332,6 +350,14 @@ class ContentMigrationFirstStage(Stage):
                                 remote=None,
                                 deferred_download=False)
                             d_artifacts.append(da)
+
+                    if missing_artifact:
+                        _logger.warn(_(
+                            'On_demand content cannot be migrated without a remote '
+                            'pulp2 unit_id: {}'.format(pulp2content.pulp2_id))
+                        )
+                        continue
+
                     for lce in pulp2lazycatalog:
                         lce.is_migrated = True
                     future_relations.update({'lces': list(pulp2lazycatalog)})
