@@ -541,7 +541,7 @@ class Pulp2Modulemd(Pulp2to3Content):
 
     artifacts = JSONField()
     checksum = models.TextField()
-    dependencies = JSONField()
+    dependencies = JSONField(default=list)
 
     pulp2_type = 'modulemd'
 
@@ -587,7 +587,7 @@ class Pulp2Modulemd(Pulp2to3Content):
                 version=md.version,
                 context=md.context,
                 arch=md.arch,
-                dependencies=md.dependencies,
+                dependencies=md.dependencies or [],
                 artifacts=md.artifacts,
                 checksum=md.checksum,
                 pulp2content=pulp2_id_obj_map[md.id])
@@ -612,7 +612,7 @@ class Pulp2ModulemdDefaults(Pulp2to3Content):
     # Unit key fields
     module = models.TextField()
     stream = models.TextField()
-    profiles = JSONField(dict)
+    profiles = JSONField(default=dict)
     digest = models.TextField()
     repo_id = models.TextField()
 
@@ -660,14 +660,20 @@ class Pulp2ModulemdDefaults(Pulp2to3Content):
         pulp2_id_obj_map = {pulp2content.pulp2_id: pulp2content for pulp2content in content_batch}
         pulp2_ids = pulp2_id_obj_map.keys()
         pulp2_content_batch = pulp2_models.ModulemdDefaults.objects.filter(id__in=pulp2_ids)
-        pulp2defaults_to_save = [
-            cls(module=defaults.name,
-                stream=defaults.stream,
-                profiles=_get_profiles(defaults.profiles),
-                digest=defaults.checksum,
-                repo_id=defaults.repo_id,
-                pulp2content=pulp2_id_obj_map[defaults.id])
-            for defaults in pulp2_content_batch]
+        pulp2defaults_to_save = []
+
+        for defaults in pulp2_content_batch:
+            kwargs = {
+                'module': defaults.name,
+                'digest': defaults.checksum,
+                'repo_id': defaults.repo_id,
+                'pulp2content': pulp2_id_obj_map[defaults.id]
+            }
+            if defaults.stream:
+                kwargs['stream'] = defaults.stream
+            if defaults.profiles:
+                kwargs['profiles'] = _get_profiles(defaults.profiles)
+            pulp2defaults_to_save.append(cls(**kwargs))
         cls.objects.bulk_create(pulp2defaults_to_save, ignore_conflicts=True,
                                 batch_size=DEFAULT_BATCH_SIZE)
 
@@ -735,7 +741,7 @@ class Pulp2Distribution(Pulp2to3Content):
         Create a Pulp 3 Distribution content for saving later in a bulk operation.
 
         Returns:
-            dict: a serialized treeinfo data
+            dict: a serialized treeinfo data, or None if no treeinfo can be found.
 
         """
         namespaces = [".treeinfo", "treeinfo"]
@@ -762,18 +768,24 @@ class Pulp2Distribution(Pulp2to3Content):
             treeinfo_serialized['distribution_tree']['build_timestamp'] = 0
             return treeinfo_serialized
 
+        return None
+
     def create_pulp3_content(self):
         """
         Create a Pulp 3 Distribution content for saving later in a bulk operation.
 
         Returns:
             (DistributionTree, dict): an in-memory DistributionTree content and serialized
-                                      treeinfo data
+                                      treeinfo data. Returns (None, None) if no treeinfo
+                                      can be found.
 
         """
         treeinfo_serialized = self.get_treeinfo_serialized()
-        return (DistributionTree(**treeinfo_serialized["distribution_tree"]),
-                treeinfo_serialized)
+        if treeinfo_serialized is None:
+            return None, None
+        else:
+            return (DistributionTree(**treeinfo_serialized["distribution_tree"]),
+                    treeinfo_serialized)
 
 
 class Pulp2PackageLangpacks(Pulp2to3Content):
@@ -835,8 +847,8 @@ class Pulp2PackageGroup(Pulp2to3Content):
     description = models.TextField()
     # This field contains mandatory, default, optional, conditional packages
     packages = JSONField()
-    desc_by_lang = JSONField(dict)
-    name_by_lang = JSONField(dict)
+    desc_by_lang = JSONField(default=dict)
+    name_by_lang = JSONField(default=dict)
     biarch_only = models.BooleanField(default=False)
 
     pulp2_type = 'package_group'
@@ -905,8 +917,8 @@ class Pulp2PackageCategory(Pulp2to3Content):
     name = models.TextField()
     description = models.TextField()
     packagegroupids = JSONField()
-    desc_by_lang = JSONField(dict)
-    name_by_lang = JSONField(dict)
+    desc_by_lang = JSONField(default=dict)
+    name_by_lang = JSONField(default=dict)
 
     pulp2_type = 'package_category'
 
