@@ -73,7 +73,12 @@ class RpmDistributor(Pulp2to3Distributor):
             if pulp2_checksum_type:
                 checksum_types = {'metadata': pulp2_checksum_type,
                                   'package': pulp2_checksum_type}
-            publish(repo_version.pk, checksum_types=checksum_types)
+            sqlite = pulp2_config.get('generate_sqlite', False)
+            try:
+                publish(repo_version.pk, checksum_types=checksum_types, sqlite_metadata=sqlite)
+            except TypeError:
+                # hack, pulp_rpm <3.9 doesn't support sqlite_metadata kwarg
+                publish(repo_version.pk, checksum_types=checksum_types)
             publication = repo_version.publication_set.first()
 
         # create distribution
@@ -101,6 +106,9 @@ class RpmDistributor(Pulp2to3Distributor):
             bool: True, if a publication needs to be recreated; False if no changes are needed
 
         """
+        if not pulp2distributor.pulp3_publication:
+            return True
+
         new_checksum_type = pulp2distributor.pulp2_config.get('checksum_type')
         current_checksum_type = pulp2distributor.pulp3_publication.cast().metadata_checksum_type
 
