@@ -94,7 +94,10 @@ def pre_migrate_content_type(content_model, mutable_type, lazy_type, premigrate_
         pulp2_content_ids = premigrate_hook()
         query_args["id__in"] = pulp2_content_ids
 
-    mongo_content_qs = content_model.pulp2.objects(_last_updated__gte=last_updated, **query_args)
+    mongo_content_qs = content_model.pulp2.objects(
+        _last_updated__gte=last_updated, **query_args
+    ).order_by("_last_updated")
+
     total_content = mongo_content_qs.count()
     _logger.debug('Total count for {type} content to migrate: {total}'.format(
         type=content_type,
@@ -136,7 +139,9 @@ def pre_migrate_content_type(content_model, mutable_type, lazy_type, premigrate_
     mongo_fields = set(['id', '_storage_path', '_last_updated', '_content_type_id'])
     if hasattr(content_model.pulp2, 'downloaded'):
         mongo_fields.add('downloaded')
-    for i, record in enumerate(mongo_content_qs.only(*mongo_fields).no_cache()):
+
+    batched_mongo_content_qs = mongo_content_qs.only(*mongo_fields).batch_size(batch_size)
+    for i, record in enumerate(batched_mongo_content_qs.no_cache()):
         if record._last_updated == last_updated:
             # corner case - content with the last``last_updated`` date might be pre-migrated;
             # check if this content is already pre-migrated
@@ -217,61 +222,6 @@ def pre_migrate_content_type(content_model, mutable_type, lazy_type, premigrate_
             pulp2content.clear()
             existing_count = 0
 
-    # If a content type needs to be associated per-repo, we need to make sure that the
-    # existing content hasn't been associated with a new repo since our last migration,
-    # and if so, we need to go back and create a Pulp2Content for these new relations.
-    if set_pulp2_repo and last_updated:
-        # last_updated is a unix timestamp that sometimes defaults to 0 (if there is no
-        # existing content). If there's no existing content we can just skip this entire
-        # process; otherwise we need to convert it to use in our Django query.
-        last_updated = datetime.utcfromtimestamp(last_updated)
-
-        # This content requires to set pulp 2 repo. E.g. for errata, because 1 pulp2
-        # content unit is converted into N pulp3 content units and repo_id is the only
-        # way to have unique records for those.
-        content_relations = Pulp2RepoContent.objects.filter(
-            pulp2_content_type_id=record._content_type_id,
-            pulp2_repository__not_in_plan=False,
-            pulp2_created__gte=last_updated
-        ).select_related(
-            'pulp2_repository'
-        ).only(
-            'pulp2_repository'
-        )
-
-        mongo_content_qs = content_model.pulp2.objects(
-            id__in=content_relations.values_list('pulp2_unit_id', flat=True))
-        pulp2_content_by_id = {
-            record.id: record for record in mongo_content_qs.only(*mongo_fields).no_cache()
-        }
-
-        for relation in content_relations:
-            record = pulp2_content_by_id[relation.pulp2_unit_id]
-            downloaded = record.downloaded if hasattr(record, 'downloaded') else False
-            item = Pulp2Content(
-                pulp2_id=record.id,
-                pulp2_content_type_id=record._content_type_id,
-                pulp2_last_updated=record._last_updated,
-                pulp2_storage_path=record._storage_path,
-                downloaded=downloaded,
-                pulp2_repo=relation.pulp2_repository
-            )
-            _logger.debug(
-                'Add content item to the list to migrate: {item}'.format(item=item))
-            pulp2content.append(item)
-            pulp2content_pb.total += 1
-            pulp2detail_pb.total += 1
-
-        pulp2content_batch = Pulp2Content.objects.bulk_create(pulp2content,
-                                                              ignore_conflicts=True)
-        pulp2content_pb.done += len(pulp2content_batch)
-        pulp2content_pb.save()
-
-        content_model.pulp_2to3_detail.pre_migrate_content_detail(pulp2content_batch)
-
-        pulp2detail_pb.done += len(pulp2content_batch)
-        pulp2detail_pb.save()
-
     pulp2content_pb.save()
     pulp2detail_pb.save()
 
@@ -326,20 +276,25 @@ def pre_migrate_lazycatalog(content_type):
         Pulp2LazyCatalog.objects.bulk_create(pulp2lazycatalog, ignore_conflicts=True)
 
 
-def pre_migrate_all_without_content(plan, type_to_repo_ids, repo_id_to_type):
+def pre_migrate_all_without_content(plan):
     """
     Pre-migrate repositories, relations to their contents, importers and distributors.
 
-    NOTE: MongoDB and Django handle datetime fields differently. MongoDB doesn't care about
-    timezones and provides "naive" time, while Django is complaining about time without a timezone.
-    The problem is that naive time != time with specified timezone, that's why all the time for
-    MongoDB comparisons should be naive and all the time for Django/PostgreSQL should be timezone
-    aware.
+    Look at the last updated times in the pulp2to3 tables for repositories/importers/distributors:
+     * pulp2_last_unit_added or pulp2_last_unit_removed for repositories
+     * pulp2_last_updated for importers and distributors
+
+    Query empty-never-had-content repos (can't filter them out in any way) and repos for which
+    there were:
+     * content changes since the last run
+     * importer changes since the last run
+     * distributor changes since the last run
+
+    Query in order of last_unit_added for the case when pre-migration is interrupted before we are
+    done with repositories.
 
     Args:
         plan(MigrationPlan): A Migration Plan
-        type_to_repo_ids(dict): A mapping from a pulp 2 repo type to a list of pulp 2 repo_ids
-        repo_id_to_type(dict): A mapping from a pulp 2 repo_id to pulp 2 repo type
     """
 
     _logger.debug('Pre-migrating Pulp 2 repositories')
@@ -349,32 +304,73 @@ def pre_migrate_all_without_content(plan, type_to_repo_ids, repo_id_to_type):
 
         for plugin_plan in plan.get_plugin_plans():
             repos = plugin_plan.get_repositories()
-            # filter by repo type
-            repos_to_check = type_to_repo_ids[plugin_plan.type]
-
-            mongo_repo_q = mongo_Q(repo_id__in=repos_to_check)
-            mongo_repo_qs = Repository.objects(mongo_repo_q)
-
-            pb.total += mongo_repo_qs.count()
-            pb.save()
-
             importers_repos = plugin_plan.get_importers_repos()
             distributors_repos = plugin_plan.get_distributors_repos()
 
-            distributor_migrators = plugin_plan.migrator.distributor_migrators
             importer_types = list(plugin_plan.migrator.importer_migrators.keys())
+            distributor_migrators = plugin_plan.migrator.distributor_migrators
+            distributor_types = list(distributor_migrators.keys())
+
+            # figure out which repos/importers/distributors have been updated since the last run
+            epoch = datetime.utcfromtimestamp(0)
+            repo_type_q = Q(pulp2_repo_type=plugin_plan.type)
+            imp_type_q = Q(pulp2_type_id__in=importer_types)
+            dist_type_q = Q(pulp2_type_id__in=distributor_types)
+
+            plugin_pulp2repos = Pulp2Repository.objects.filter(repo_type_q)
+            repo_premigrated_last_by_added = plugin_pulp2repos.aggregate(
+                Max('pulp2_last_unit_added')
+            )['pulp2_last_unit_added__max'] or epoch
+            repo_premigrated_last_by_removed = plugin_pulp2repos.aggregate(
+                Max('pulp2_last_unit_removed')
+            )['pulp2_last_unit_removed__max'] or epoch
+            imp_premigrated_last = Pulp2Importer.objects.filter(imp_type_q).aggregate(
+                Max('pulp2_last_updated')
+            )['pulp2_last_updated__max'] or epoch
+            dist_premigrated_last = Pulp2Distributor.objects.filter(dist_type_q).aggregate(
+                Max('pulp2_last_updated')
+            )['pulp2_last_updated__max'] or epoch
+
+            is_content_added_q = mongo_Q(last_unit_added__gte=repo_premigrated_last_by_added)
+            is_content_removed_q = mongo_Q(last_unit_removed__gte=repo_premigrated_last_by_removed)
+            is_new_enough_repo_q = is_content_added_q | is_content_removed_q
+            is_empty_repo_q = mongo_Q(last_unit_added__exists=False)
+            is_new_enough_imp_q = mongo_Q(last_updated__gte=imp_premigrated_last)
+            is_new_enough_dist_q = mongo_Q(last_updated__gte=dist_premigrated_last)
+            repo_repo_id_q = mongo_Q(repo_id__in=repos)
+            imp_repo_id_q = mongo_Q(repo_id__in=importers_repos)
+            dist_repo_id_q = mongo_Q(repo_id__in=distributors_repos)
+
+            updated_importers = Importer.objects(
+                imp_repo_id_q & is_new_enough_imp_q
+            ).only('repo_id')
+            updated_imp_repos = set(imp.repo_id for imp in updated_importers)
+            updated_distributors = Distributor.objects(
+                dist_repo_id_q & is_new_enough_dist_q
+            ).only('repo_id')
+            updated_dist_repos = set(dist.repo_id for dist in updated_distributors)
+            updated_impdist_repos = updated_imp_repos | updated_dist_repos
+
+            mongo_updated_repo_q = repo_repo_id_q & (is_new_enough_repo_q | is_empty_repo_q)
+            mongo_updated_imp_dist_repo_q = mongo_Q(repo_id__in=updated_impdist_repos)
+
+            mongo_repo_qs = Repository.objects(
+                mongo_updated_repo_q | mongo_updated_imp_dist_repo_q
+            ).order_by('last_unit_added')
+
+            pb.total += mongo_repo_qs.count()
+            pb.save()
 
             for repo_data in mongo_repo_qs.only('id',
                                                 'repo_id',
                                                 'last_unit_added',
                                                 'last_unit_removed',
-                                                'description',
-                                                'notes'):
+                                                'description'):
                 repo = None
                 repo_id = repo_data.repo_id
                 with transaction.atomic():
                     if not repos or repos and repo_id in repos:
-                        repo = pre_migrate_repo(repo_data, repo_id_to_type)
+                        repo = pre_migrate_repo(repo_data, plan.repo_id_to_type)
                     # do not pre-migrate importers/distributors in case of special repo setup
                     # and no importers/distributors were specified in the MP
                     if not repos or repos and importers_repos:
@@ -382,13 +378,19 @@ def pre_migrate_all_without_content(plan, type_to_repo_ids, repo_id_to_type):
                     if not repos or repos and distributors_repos:
                         pre_migrate_distributor(
                             repo_id, distributors_repos, distributor_migrators, repo)
-                pb.increment()
+                    pb.increment()
 
 
 def pre_migrate_repo(record, repo_id_to_type):
     """
     Pre-migrate a pulp 2 repo.
 
+    NOTE: MongoDB and Django handle datetime fields differently. MongoDB doesn't care about
+    timezones and provides "naive" time, while Django is complaining about time without a timezone.
+    The problem is that naive time != time with specified timezone, that's why all the time for
+    MongoDB comparisons should be naive and all the time for Django/PostgreSQL should be timezone
+    aware.
+
     Args:
         record(Repository): Pulp 2 repository data
         repo_id_to_type(dict): A mapping from a pulp 2 repo_id to pulp 2 repo types
@@ -611,7 +613,7 @@ def pre_migrate_repocontent(repo):
     Pulp2RepoContent.objects.bulk_create(repocontent, batch_size=DEFAULT_BATCH_SIZE)
 
 
-def handle_outdated_resources(plan, type_to_repo_ids):
+def handle_outdated_resources(plan):
     """
     Marks repositories, importers, distributors which are no longer present in Pulp2.
 
@@ -619,13 +621,12 @@ def handle_outdated_resources(plan, type_to_repo_ids):
 
     Args:
         plan(MigrationPlan): A Migration Plan
-        type_to_repo_ids(dict): A mapping from a pulp 2 repo type to a list of pulp 2 repo_ids
     """
     for plugin_plan in plan.get_plugin_plans():
         inplan_repos = plugin_plan.get_repositories()
 
         # filter by repo type
-        repos_to_consider = type_to_repo_ids[plugin_plan.type]
+        repos_to_consider = plan.type_to_repo_ids[plugin_plan.type]
 
         # in case only certain repositories are specified in the migration plan
         if inplan_repos:
@@ -669,7 +670,7 @@ def handle_outdated_resources(plan, type_to_repo_ids):
     # Delete old Publications/Distributions which are no longer present in Pulp2.
 
     # It's critical to remove Distributions to avoid base_path overlap.
-    # It make the migration logic easier if we remove old Publications as well.
+    # It makes the migration logic easier if we remove old Publications as well.
 
     # Delete criteria:
     #     - pulp2distributor is no longer in plan
