@@ -1,4 +1,5 @@
 import json
+import time
 import unittest
 
 from pulpcore.client.pulpcore import (
@@ -18,13 +19,14 @@ from pulpcore.client.pulp_2to3_migration import (
 )
 from pulp_2to3_migration.tests.functional.util import (
     get_psql_smash_cmd,
-    monitor_task
+    set_pulp2_snapshot
 )
 
 from pulp_smash import cli
 from pulp_smash import config as smash_config
+from pulp_smash.pulp3.bindings import monitor_task, monitor_task_group
 
-from .constants import TRUNCATE_TABLES_QUERY_BASH
+from .constants import BINDINGS_CONFIGURATION, TRUNCATE_TABLES_QUERY_BASH
 
 PULP_2_ISO_FIXTURE_DATA = {
     'file': 3,
@@ -106,11 +108,7 @@ class TestMigrationPlan(unittest.TestCase):
         """
         Create all the client instances needed to communicate with Pulp.
         """
-        configuration = Configuration()
-        configuration.username = 'admin'
-        configuration.password = 'password'
-        configuration.host = 'http://pulp'
-        configuration.safe_chars_for_path_param = '/'
+        configuration = Configuration(**BINDINGS_CONFIGURATION)
 
         core_client = CoreApiClient(configuration)
         file_client = FileApiClient(configuration)
@@ -123,18 +121,22 @@ class TestMigrationPlan(unittest.TestCase):
         cls.tasks_api = TasksApi(core_client)
         cls.migration_plans_api = MigrationPlansApi(migration_client)
 
+        set_pulp2_snapshot(name='file_base_4repos')
+
     def tearDown(self):
         """
         Clean up the database after each test
         """
         cmd = get_psql_smash_cmd(TRUNCATE_TABLES_QUERY_BASH)
         self.smash_cli_client.run(cmd, sudo=True)
+        time.sleep(0.5)
 
     def _do_test(self, repos, migration_plan):
         mp = self.migration_plans_api.create({'plan': migration_plan})
         mp_run_response = self.migration_plans_api.run(mp.pulp_href, {})
-        task = monitor_task(self.tasks_api, mp_run_response.task)
-        self.assertEqual(task.state, "completed")
+        task = monitor_task(mp_run_response.task)
+        monitor_task_group(task.task_group)
+
         for repo_id in repos:
             pulp3repos = self.file_repo_api.list(name=repo_id)
             # Assert that there is a result
